/*
CHANNEL 1...N
*/
/////////////////////////////////////////
//

#include "sharedmem.h"

#include <map>
using namespace std;
#include "../errlog.h"
#include <stdio.h>

extern const unsigned int SM_CHANNEL_SIZE	= sizeof(CChannel);
extern const unsigned int SM_CHANNEL_COUNT	= 1024;
static void *_MEM = 0;

#ifdef _WIN32
#include <windows.h>

#define APPL_MEM_MEPPER_NAME "PVWEB_IPC_CHANNELS"
HANDLE mapH = 0;

extern "C" void InitSharedMem()
{
	string mappedfile_name = APPL_MEM_MEPPER_NAME;
	char *content = getenv( "PVWEB_PROXY_COMM_FILE" );
	if(content && strlen( content ) )
	{
	    mappedfile_name = content;
	}
	log_error( LOG_LEVEL_INFO, "Using memory mapped file object %s", mappedfile_name.c_str( ) );
	
	mapH = CreateFileMapping(
		(HANDLE)INVALID_HANDLE_VALUE, // fHandle,                      // handle to file to map
		0,                            // optional security attributes
		PAGE_READWRITE | SEC_COMMIT,
		// protection for mapping object
		0x00000000,                   // high-order 32 bits of object size
		SM_CHANNEL_SIZE*SM_CHANNEL_COUNT, // low-order 32 bits of object size
		mappedfile_name.c_str( )          // name of file-mapping object
		);
	if( mapH ) {
		_MEM = MapViewOfFile(
			mapH,                          // file-mapping object to map into
			// address space
			FILE_MAP_ALL_ACCESS,      // access mode
			0,     // high-order 32 bits of file offset
			0x00000000,      // low-order 32 bits of file offset
			0x00000000  // number of bytes to map				
			);
		if( !_MEM ) {
			LPVOID lpMsgBuf;
			FormatMessage(
				FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
				NULL,
				GetLastError(),
				MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
				(LPTSTR) &lpMsgBuf,
				0,
				NULL
				);
			if( lpMsgBuf ) {

#if defined( _OBWEB ) || defined( _OBMAIL )
				CbLogMsg( LOG_ERROR, (char *)lpMsgBuf );
				LocalFree( lpMsgBuf );
				CbLogMsg( LOG_ERROR | LOG_FATAL, "Aborting!" );
#else
				::MessageBox( 0, (char *)lpMsgBuf,"error", MB_OK );
				LocalFree( lpMsgBuf );
#endif
				exit( 5 );

			}
		}
		memset( _MEM, 0x00, SM_CHANNEL_SIZE*SM_CHANNEL_COUNT );
	}
	else {
#if defined( _OBWEB ) || defined( _OBMAIL )
		CbLogMsg( LOG_ERROR | LOG_FATAL, string( "Could not create filemapping for " ) + mappedfile_name + "! Aborting" );
#else
		string msg = "Could not create filemapping for ";
		msg += mappedfile_name + "Aborting";
		::MessageBox( 0, msg.c_str( ),"error", MB_OK );
#endif
		exit( 5 );
	}
}

extern "C" void DeInitSharedMem()
{
	if( _MEM ) UnmapViewOfFile( _MEM );
	_MEM = NULL;
    if( mapH ) CloseHandle( mapH );
	mapH = 0;
}


#endif // CbWIN32

#ifdef CbLINUX

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <stdlib.h>
#include "support/filesupport.h"

int mapH = 0;
key_t SM_KEY = 5580;

extern "C" void InitSharedMem()
{
	char *content = getenv( "PVWEB_PROXY_COMM_FILE" );
	if(content && strlen( content ) )
	{
	    SM_KEY = atoi(content);
	}

	char tmp[1024];
	sprintf(tmp, "Using memory mapped file object %i", (int)SM_KEY);
	log_error( LOG_LEVEL_INFO, tmp );

	if ((mapH = shmget(SM_KEY, SM_CHANNEL_SIZE*SM_CHANNEL_COUNT, IPC_CREAT | 0666)) < 0) {
			perror("Server InitSharedMem failed" );
			log_error( LOG_LEVEL_FATAL, "Cannot Init SharedMEM... Aborting");
			exit( 5 );
	}
	else
	{
		if ((_MEM = shmat(mapH, NULL, 0)) == (char *) -1) {
				perror("Server attach SharedMem");
				log_error( LOG_LEVEL_FATAL, "Cannot Attach SharedMEM... Aborting");
				exit(5);
		}
	}
}

extern "C" void DeInitSharedMem()
{
	if( _MEM ) shmdt(_MEM);
	_MEM = NULL;
}

#endif

extern void * GetSharedMem( int offset )
{
	return (char *)(_MEM) + (offset * SM_CHANNEL_SIZE);
}