/*
CHANNEL 1...N
*/
/////////////////////////////////////////
//
#include "../jcc.h"

extern "C" {
#include "../icap/src/iserver.h"
}

#include "sharedmem.h"
#include "clientchannel.h"
#include "obwebcomm.h"
#include <stdio.h>

#include "cbsync.h"

#define FUNC_CHECK_URL	"chkurl"
#define FUNC_AUTH		"auth"
#define FUNC_GET_PRX_CONFIG	"getprxcfg"
#define FUNC_GET_ICAP_CONFIG	"geticapcfg"

#ifdef CbWIN32
extern "C" SOCKET srv_socket;
#else
extern "C" int srv_socket;
#endif

extern void CbSleep( unsigned int );
extern "C" void ApplyConfig( iserver_info_t *cfg );
extern "C" int G_ICAP_SHUTDOWN;
#include <string>
using namespace std;

extern"C" int restartNeeded(iserver_info_t _icap_server_settings );
extern "C" int updateNeeded(iserver_info_t _icap_server_settings );

extern "C" char* StripHTTPHeader( char * in )
{
	string html = in;
	free(in);
	if( !html.empty() )
    {
       	unsigned int pos = html.find( "\r\n\r\n");
       	if( pos != string::npos) {
	      	pos = pos + 4;
        	char* result;
	       	result = (char*)malloc(html.length()+1);
        	strcpy( result, html.substr( pos ).c_str( ) );
  	        return result;
        }
    }
    
	return 0;
}

extern "C" char* OBWebCheckURL( const char* ip, const char* url, const char* user )
{
	try {
		string answer;

		IPC* _ipc = new IPC;

		string data;
		data += FUNC_CHECK_URL;
		data += "\t";
		if( ip && strlen(ip) ) data += ip;
		data += "\t";
		if( url && strlen(url) ) data += url;
		data += "\t";
		if( user && strlen(user) ) data += user;

		
		
		answer = _ipc->Send( data );

		delete _ipc;
		_ipc = 0;

		if( answer.empty()) return 0;
		return strdup( answer.c_str() );
	}
	catch(...) {}
	return 0;

}

extern "C" int AuthUser(const char* username, const char* password, char** error)
{
	try {
		IPC* _ipc = new IPC;

		string data;
		data += FUNC_AUTH;
		data += "\t";
		if( username && strlen(username) ) data += username;
		data += "\t";
		if( password && strlen(password) ) data += password;
		string answer = _ipc->Send( data );
		delete _ipc;
		_ipc = 0;
		*error = 0;

		if( answer.empty()) return 0;

		int result = 0;
		int pos = answer.find( "\t" );		
		if( pos != string::npos ) {
			result = atoi( answer.substr( 0, pos ).c_str() );
			*error = strdup( answer.substr( pos + 1 ).c_str() );
		}
		return result;
	}
	catch(...) {}
	return 0;
}

extern "C" void GetG_Config(GConfig* newconfig)
{
    *newconfig = G_Proxy_Config;	
}

extern "C" int g_terminate;

extern "C" void CheckForNewProxyConfig()
{
	static int old_port = -1;
	static int old_enabled = -1;

	try {
		
		GConfig newconfig;
		IPC* _ipc = new IPC;
		string data;
		data += FUNC_GET_PRX_CONFIG;
		string answer = _ipc->Send( data );
		memcpy( &newconfig, (GConfig*)answer.begin( ), sizeof( GConfig ));
		delete _ipc;
		_ipc = 0;


		newconfig._UPDATE  = 0;
		int modified = memcmp( &G_Proxy_Config, &newconfig, sizeof(GConfig));
		newconfig._UPDATE = modified ? 1: 0;
		G_Proxy_Config = newconfig;
	}
	catch(...) {}

	
	if( old_enabled != -1 )
	{
		if ( old_enabled != G_Proxy_Config._ENABLED ) {
			g_terminate = 1;
			dummy_request( old_port );	
		}
	}
		old_enabled = G_Proxy_Config._ENABLED;
	
	if( old_port != -1 )
		if( old_port != G_Proxy_Config._PORT )
			dummy_request( old_port );
	old_port = G_Proxy_Config._PORT;
}

extern "C" void CheckForNewProxyConfigThread()
{
	while(1)
	{
		CheckForNewProxyConfig();
#ifdef _DEBUG
		CbSleep(1000);
#else
		CbSleep(30000);
#endif
	}
}

extern "C" void threadpool_shutdown(void);

extern "C" 
void IcapGetNewConfig()
{

	static int old_stat_enabled = -1;
#if 1
	try {
		
		iserver_info_t newconfig;
		IPC* _ipc = new IPC;
		string data;
		data += FUNC_GET_ICAP_CONFIG;
		string answer = _ipc->Send( data );
		memcpy( &newconfig, (iserver_info_t*)answer.begin( ), sizeof( iserver_info_t ));
		delete _ipc;
		_ipc = 0;
		if( updateNeeded( newconfig ) || restartNeeded( newconfig ) || old_stat_enabled != newconfig.stat_enabled ) {
			if( restartNeeded( newconfig ) == 1 ) {
				G_ICAP_SHUTDOWN = 1;
#ifndef CbLINUX				
				if (srv_socket != INVALID_SOCKET)
					closesocket( srv_socket );
#else
				dummy_request( cinfo.opt_port );
#endif
			}
			if( newconfig.stat_enabled == 0 ) 
			{
				G_ICAP_SHUTDOWN = 1;
#ifndef CbLINUX				
				if (srv_socket != INVALID_SOCKET)
					closesocket( srv_socket );
#else
				dummy_request( cinfo.opt_port );
#endif
			}
			ApplyConfig( &newconfig );
			old_stat_enabled = newconfig.stat_enabled;
		}
	}
	catch(...) {
		printf("exeption: obwebcomm.cpp:IcapGetNewConfig");
	}
	
#endif	
}

extern "C" void CheckForNewIcapConfigThread()
{
	while(1)
	{
#ifdef _DEBUG
		CbSleep(1000);
#else
		CbSleep(30000);
#endif
		IcapGetNewConfig();
	}
}

extern "C" int IsICAPEnabled(void)
{
	return cinfo.stat_enabled;
}

extern "C" char* GetLogDir(void)
{
	return cinfo.logpath;
}
