
/*********************************************************************
 *********************************************************************/

#include "config.h"

#include <stdio.h>
#include <sys/types.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <assert.h>

#ifndef _WIN32
#ifndef __OS2__
#include <unistd.h>
#endif /* ndef __OS2__ */
#include <netinet/in.h>
#else
#include <winsock2.h>
#endif /* ndef _WIN32 */

#ifdef __OS2__
#include <utils.h>
#endif /* def __OS2__ */

#include "../project.h"
#include "../filters.h"
#include "../encode.h"
#include "../parsers.h"
#include "../ssplit.h"
#include "../errlog.h"
#include "../jbsockets.h"
#include "../miscutil.h"
#include "../actions.h"
#include "../cgi.h"
#include "../list.h"
#include "../deanimate.h"
#include "../urlmatch.h"

#ifdef _WIN32
#include "../win32.h"
#endif

extern char * uudecode(const char* bufcoded);

/* Fix a problem with Solaris.  There should be no effect on other
 * platforms.
 * Solaris's isspace() is a macro which uses it's argument directly
 * as an array index.  Therefore we need to make sure that high-bit
 * characters generate +ve values, and ideally we also want to make
 * the argument match the declared parameter type of "int".
 */
#define ijb_isdigit(__X) isdigit((int)(unsigned char)(__X))

extern char* StripHTTPHeader( char * in );
extern char* OBWebCheckURL( const char* ip, const char* url, const char* user );
extern int AuthUser(const char* username, const char* password, char** error);
/*********************************************************************
 *
 * Function    :  block_url
 *
 * Description :  Called from `chat'.  Check to see if we need to block this.
 *
 * Parameters  :
 *          1  :  csp = Current client state (buffers, headers, etc...)
 *
 * Returns     :  NULL => unblocked, else HTTP block response
 *
 *********************************************************************/
 struct http_response *obweb_block_url(struct client_state *csp)
{
   struct http_response *rsp;
   char* ip_addr;
   char* url;
   char* response;
   char* user;

   /*
	* If it's not blocked, don't block it ;-)
	*/

	{

		ip_addr = csp->ip_addr_str;
	   url = csp->http->url;
	   user = csp->user; 
	   response = OBWebCheckURL( ip_addr, url, user );
	   if( !response )
		   return 0;

	   /*
		* Else, prepare a response
		*/

	   StripHTTPHeader( response );
	   if (NULL == (rsp = alloc_http_response()))
	   {
		  return cgi_error_memory();
	   }
	   rsp->status = strdup("403 Forbidden");
	   rsp->body = response;

	   return finish_http_response(rsp);
	}

}

int
authenticate_basic( char* cleartext, char** usr)
{
	char* username;
	char* passwd;
	char* error_msg;
	int result;
	strtok(cleartext, "\r\n");

	username = strdup(cleartext);
	free(cleartext);

	if ((cleartext = strchr(username, ':')) != NULL)
		 *(cleartext)++ = '\0';

	passwd = cleartext;
	if (cleartext == NULL)
		 return -1;
		 
	*usr = strdup(username);
 	
	result = 0;
	
	result = AuthUser(username, passwd, &error_msg);
	
	   if (!result)
	{
		log_error(LOG_LEVEL_INFO, "[207111]proxy authentication failed: %s User: %s", error_msg, username);
		free(error_msg);
	  }

	free( username );

	return result;
}

/*********************************************************************
 *
 *********************************************************************/
struct http_response *check_authentication(struct client_state *csp)
{
   char* p;
   struct http_response *rsp;
   char* cleartext;
   char* sent_auth;
   char* username;
   int auth_ok = 0;

   p = get_header_value(csp->headers, "Proxy-Authorization:");
   if( !p ) {
	auth_ok = 0;
   }
   else {
	if( 0 != (p = strstr(p, "Basic")) ) {
		 while (!isspace(*p))
			  p++;
		 /* Trim leading whitespace before decoding */
		 while (isspace(*p))
			  p++;
		 /* username and password */
		 sent_auth = strdup(p);
		 /* Trim trailing \n before decoding */
		 strtok(sent_auth, "\n");
		 cleartext = uudecode(sent_auth);
		 free(sent_auth);
		 auth_ok = authenticate_basic( cleartext, &username);
		 csp->user = strdup(username);
		 free(username);
		 if( auth_ok )
			;//free(p); // why? no allocation of memory, just a plain pointer
	}
  }

  if( auth_ok )
  {
	return obweb_block_url(csp);
  }


   /*
	* Else, prepare a response
	*/
   if (NULL == (rsp = alloc_http_response()))
   {
	  return cgi_error_memory();
   }

   rsp->status = strdup("407 Proxy Authentication Required");
   enlist(rsp->headers, "Proxy-Authenticate: Basic realm=Proxy");

   return finish_http_response(rsp);

}
