
#include <stdio.h>

#include "cobion/mmf.h"

#include <stdio.h>
#include <time.h>

#ifdef CbWIN32
#include <process.h>
#else
#include <unistd.h>
#include <pthread.h>
#endif

extern "C" int G_ICAP_SHUTDOWN;
extern "C" int proxy_main( int, char** );

extern void CbSleep( unsigned int );

extern "C" char* OBWebCheckURL( const char* ip, const char* url, const char* user );
extern "C"
char* IcapReqModCheckUrl( const char* ip
                        , const char* user
                        , const char* url )
{
	return OBWebCheckURL( ip, url, user );
}
// ----------------------------------------------------------

FILE *out = NULL;

extern "C" char *GetLogDir();

void initIcapLogfile()
{
	if( out )
		return;

	std::string filename;
	filename = GetLogDir();
	filename += "icap.log";
	out = fopen( filename.c_str(), "a" );
}

char *get_date(void)
{
  time_t tp;
  char *ascitime;
  char *s;
  
  tp = time(NULL);
  
  ascitime = ctime(&tp);
  
  s = (char *)malloc(sizeof(char) * (strlen(ascitime)+1));
  if(s == NULL)
    exit(3); /* no use writing an error message, because this function
                will keep getting called! */
  strcpy(s, ascitime);
  s[strlen(ascitime) - 1] = '\0';
  
  return s;
}

extern "C"
void IcapLogMsg( const char* msg, int error, int level ) 
{
		char * time;
		time = get_date();

		if( out ) {
			switch( error )
			{
			case 0:
				fprintf( out, "%s\tICAP-INFO:\t%s\n", time, msg );
				break;
			case 1:
				fprintf( out, "%s\tICAP-ERROR:\t%s\n", time, msg );
				break;
			case 2:
				fprintf( out, "%s\tICAP-MESSAGE:\t%s\n", time, msg );
				break;
			case 3:
				fprintf( out, "%s\t%s\n",time, msg);
				break;
			}
		}

		fflush(out);
}
// ----------------------------------------------------------
extern "C" void IcapGetNewConfig();

/* ------------------------------------------------------------------------- */

extern "C" void* CheckForNewIcapConfigThread(void *);
extern "C" int iserver_await_connections();
extern "C" int IsICAPEnabled();


int icap_main( void )
{
	
	IcapGetNewConfig();
	initIcapLogfile();
#ifdef _WIN32
		_beginthread(
			(void (*)(void *))CheckForNewIcapConfigThread,
			64 * 1024,
			NULL);
#else
		{
		pthread_t config_thread;
		pthread_attr_t attrs;
		pthread_attr_init(&attrs);
		pthread_attr_setdetachstate(&attrs, PTHREAD_CREATE_DETACHED);
		pthread_create(&config_thread, &attrs,
		   CheckForNewIcapConfigThread, NULL);
		
		sleep(1); // if its a Daemon sleep because reading the Config File

		pthread_attr_destroy(&attrs);
		}
#endif

	while ( 1 ) {
		if( IsICAPEnabled() ) {
			G_ICAP_SHUTDOWN = 0;
			IcapLogMsg("Starting Icap Server", 3, 0 );
			iserver_await_connections();
			IcapLogMsg("Shut down Icap Server", 3, 0 );
		}
		if( G_ICAP_SHUTDOWN == 1 ) {
			G_ICAP_SHUTDOWN = 0;
		}
#ifdef _WIN32
		CbSleep( 1000 );
#else
		sleep(1);
#endif
	}
	return 0;
}

void* IcapThread(void*)
{
	while (1) {
		icap_main();
		CbSleep( 1000 );
	}
}

void* ProxyThread(void*)
{
	initIcapLogfile();
	char *argv[2] = { "proxy\0", 0 };
	while (1) {
		proxy_main( 1, argv );
		CbSleep( 1000 );
	}
}

void main(int argc, const char* argv[] )
{
	
InitSharedMem();
#ifdef _WIN32
		_beginthread(
			(void (*)(void *))IcapThread,
			64 * 1024,
			NULL);
		_beginthread(
			(void (*)(void *))ProxyThread,
			64 * 1024,
			NULL);
#else
		{
		pthread_t icap_thread;
		pthread_attr_t attrs;
		pthread_attr_init(&attrs);
		pthread_attr_setdetachstate(&attrs, PTHREAD_CREATE_DETACHED);
		pthread_create(&icap_thread, &attrs,
		   IcapThread, NULL);
		
		pthread_attr_destroy(&attrs);

		pthread_t proxy_thread;
		pthread_attr_t proxy_attrs;
		pthread_attr_init(&proxy_attrs);
		pthread_attr_setdetachstate(&proxy_attrs, PTHREAD_CREATE_DETACHED);
		pthread_create(&proxy_thread, &proxy_attrs,
		   ProxyThread, NULL);
		
		pthread_attr_destroy(&attrs);
		}
#endif
		while(1)
			CbSleep( 1000 * 100 );
	DeInitSharedMem();
}
